<?php

require_once __DIR__.'/../../synergy/models/synergy.php';

class Limited_User_Test extends PHPUnit_Framework_TestCase {

	private $config = array (
		'auth' => array (
			'common' => array (
				'default_auth' => 'mydefault',
				'session_key' => 'testkey'
			),
			'mydefault' => array (
				'driver' => 'Default'
			)
		),
		'auth_users' => array (
			'user_plain' => array (
				'username' => 'user_plain',
				'realname' => 'Just a plain user',
				'password' => 'user_pass',
				'password_algo' => 'plain',
				'modules' => array (
					'mydefault'
				),
				'groups' => array (
					'plain_group'
				)
			)
		),
		'auth_groups' => array (
			'plain_group' => array (
				'some_plain_access'
			)
		)
	);

	function setup() {
		$_SESSION = array();
		op5objstore::instance()->mock_clear();
		op5objstore::instance()->clear();
		op5objstore::instance()->mock_add('op5config', new MockConfig($this->config));
	}

	function teardown() {
		$_SESSION = array();
		op5objstore::instance()->mock_clear();
		op5objstore::instance()->clear();
	}

	/**
	 * @group MON-9191
	 */
	function test_limited_user_should_not_stack_trace_when_accessing_models() {
		$op5auth = op5auth::instance();
		$op5auth->write_close();
		$op5auth->force_user(
			new User_Model(
				array (
					'username' => 'snake',
					'realname' => 'This is forced',
					'email' => 'i_do_have@n.email',
					'auth_data' => array (
						'i_can_do_other_stuff' => true
					),
					'groups' => array (
					)
				)
			)
		);

		// mock auth_data for the logged in user to not include
		// host_view_all nor service_view_all
		$data = array(
			'result' => array(
				'host_contacts' => array(
					'snake'
				),
				'service_contacts' => array(
					'snake'
				)
			)
		);
		$auth = Auth::instance();
		$this->assertSame(true, Synergy_Model::__is_permitted_by_hosts($data, $auth));
		$this->assertSame(true, Synergy_Model::__is_permitted_by_services($data, $auth));
	}
}

